//
// sequence.cc
//
// Simple program for creating and editing a sequence of integers.
//
// When run requests commands like
//
//   append 45
//   append 18
//   append 10
//   insert 37 1
//   remove 3
//   exit
//
// in this case, to create the sequence [45,37,18] then quit.
//
// The sequences can grow to an arbitrary size.
//
#include <iostream>
#include <cassert>

struct sequence {
  int* elements; // Array of integers whose prefix is the sequence.
  int  size;     // Number of valid items in the sequence.
  int  capacity; // The allocated size of the array.
};

//
// newSequence(initialCapacity)
//
// Make a new, empty sequence whoe storage has the given initial
// capacity. Returns the pointer to it.
//
sequence* newSequence(int initialCapacity) {
  sequence* S = new sequence;
  S->elements = new int[initialCapacity];  
  S->size = 0;                     
  S->capacity = initialCapacity;
  return S;
}

//
// resize(S, newCapacity)
//
// Allocates new storage for the given sequence `S`,
// releases the old storage.
//
void resize(sequence* S, int newCapacity) {

  // PUT YOUR CODE HERE
  
}

//
// append(v,S)
//
// Puts value `v` onto the end of sequence `S`.
//
// If the current storage for `S` isn't large enough, doubles its
// capacity with `resize`.
//
void append(int v, sequence* S) {
  if (S->size >= S->capacity) {
    resize(S,2*S->capacity);
  }
  assert(S->size < S->capacity); 
  S->elements[S->size] = v;           
  S->size++;                        
}

//
// output(S)
//
// Outputs the sequence `S` within a text line.
//
void output(sequence* S) {
  
  // IMPROVE THIS CODE
  
  for (int i=0; i < S->size; i++) {
    std::cout << S->elements[i] << " ";
  }
}

//
// insertAt(i,S,v)
//
// Puts a new value `v` within `S` at the position `i`.
//
// Position 0 puts it at the front. A position of `S->size` puts it at
// the end.
//
void insertAt(int i, sequence* S, int v) {
  assert(0 <= i && i <= S->size);

  // PUT YOUR CODE HERE
  
}
  
void removeAt(int i, sequence* S) {
  assert(0 <= i && i <= S->size);

  // PUT YOUR CODE HERE

}  

//
// release(S)
//
// De-allocates the heap storage for the sequence `S`.
//
void release(sequence* S) {
  delete [] S->elements;
  delete S;
}

//
// main()
//
// Loops repeatedly, asking for commands that edit an initially empty
// sequence.
//
int main(void) {

  // Build an empty sequence with capacity of 2.
  sequence* seq = newSequence(2);

  // Get a command and process it, calling the functions above.
  std::string command;
  do {
    std::cout << "Your sequence is ";
    output(seq);
    std::cout << "." << std::endl;
    std::cout << "Choose one of these commands:" << std::endl;
    std::cout << "\tinsert" << std::endl;
    std::cout << "\tremove" << std::endl;
    std::cout << "\tappend" << std::endl;
    std::cout << "\texit" << std::endl;
    std::cout << "What's your command? ";
    std::cin >> command;
    if (command == "append" || command == "insert") {

      // Insert and append each take a number to be placed into the
      // sequence.
      //
      int number;
      std::cout << "What number would you like to " << command << "? ";
      std::cin >> number;
      if (command == "append") {
	
	append(number, seq);

      } else {

	// Get the position.
	int position;
	std::cout << "At what position? ";
	std::cin >> position;
	//
	insertAt(position, seq, number);

      }
    } else if (command == "remove") {

      // Get the position.
      int position;
      std::cout << "Which position should be deleted? ";
      std::cin >> position;
      //
      removeAt(position, seq);
    }
  } while (command != "exit");

  release(seq);
  return 0;
}
